@@ -61,6 +61,11 @@ main(
     char *exitstr;
     amwait_t status;
 
+    if (argc > 1 && argv && argv[1] && g_str_equal(argv[1], "--version")) {
+	printf("killpgrp-%s\n", VERSION);
+	return (0);
+    }
+
     /*
      * Configure program for internationalization:
      *   1) Only set the message locale for now.
