@@ -66,6 +66,11 @@ main(
     char *cmdline;
 #endif /* ERRMSG */
 
+    if (argc > 1 && argv && argv[1] && g_str_equal(argv[1], "--version")) {
+	printf("rundump-%s\n", VERSION);
+	return (0);
+    }
+
     /*
      * Configure program for internationalization:
      *   1) Only set the message locale for now.
