@@ -165,6 +165,7 @@ am_init_feature_set(void)
 	am_add_feature(f, fe_amrecover_receive_unfiltered);
 	am_add_feature(f, fe_application_client_name);
 	am_add_feature(f, fe_script_client_name);
+	am_add_feature(f, fe_dumptype_property);
     }
     return f;
 }
@@ -459,3 +460,10 @@ bad:
     am_release_feature_set(f);
     return NULL;
 }
+
+am_feature_t *
+am_features(
+    am_feature_t *f)
+{
+    return f;
+}
