@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007,2008,2009 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007, 2008, 2009, 2010 Zmanda, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -62,10 +62,16 @@ GValue* g_value_unset_init(GValue* val, GType type);
  * Returns its second (reset) argument.*/
 GValue* g_value_unset_copy(const GValue* from, GValue * to);
 
+/* This function is available in glib-2.28.0 and higher; for lower versions
+ * we build our own version with a different name */
+#if (GLIB_MAJOR_VERSION < 2 || (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 28))
+void slist_free_full(GSList * list, GDestroyNotify free_fn);
+#else
+#define slist_free_full(list, free_fn) g_slist_free_full((list), (free_fn))
+#endif
+
 /* These functions all take a GLib container, and call free() on all the
  * pointers in the container before free()ing the container itself. */
-void g_list_free_full(GList * list);
-void g_slist_free_full(GSList * list);
 void g_queue_free_full(GQueue * queue);
 void g_ptr_array_free_full(GPtrArray * array);
 
@@ -121,7 +127,7 @@ void g_ptr_array_foreach (GPtrArray *array,
 #endif
 
 /* functions for g_hash_table_new to hash and compare case-insensitive strings */
-guint g_str_case_hash(gconstpointer v);
-gboolean g_str_case_equal(gconstpointer v1, gconstpointer v2);
+guint g_str_amanda_hash(gconstpointer v);
+gboolean g_str_amanda_equal(gconstpointer v1, gconstpointer v2);
 
 #endif
