@@ -37,9 +37,6 @@
 #ifdef BSD_SECURITY
 extern const security_driver_t bsd_security_driver;
 #endif
-#ifdef KRB4_SECURITY
-extern const security_driver_t krb4_security_driver;
-#endif
 #ifdef KRB5_SECURITY
 extern const security_driver_t krb5_security_driver;
 #endif
@@ -61,9 +58,6 @@ static const security_driver_t *drivers[] = {
 #ifdef BSD_SECURITY
     &bsd_security_driver,
 #endif
-#ifdef KRB4_SECURITY
-    &krb4_security_driver,
-#endif
 #ifdef KRB5_SECURITY
     &krb5_security_driver,
 #endif
