@@ -45,12 +45,13 @@ typedef struct tapelist_s {
     char *label;
     int isafile; /* set to 1 and make *label the path to the file */
     off_t *files;
+    int   *partnum;
     int numfiles;
 } tapelist_t;
 
 int num_entries(tapelist_t *tapelist);
 tapelist_t *append_to_tapelist(tapelist_t *tapelist, char *label,
-					off_t file, int isafile);
+					off_t file, int partnum, int isafile);
 char *marshal_tapelist(tapelist_t *tapelist, int escape);
 tapelist_t *unmarshal_tapelist_str(char *tapelist_str);
 char *escape_label(char *label);
