@@ -93,8 +93,10 @@ my $getproplist;
 my $device_name;
 my $print_label;
 
+debug("Arguments: " . join(' ', @ARGV));
 Getopt::Long::Configure(qw(bundling));
 GetOptions(
+    'version' => \&Amanda::Util::version_opt,
     'help|usage|?' => \&usage,
     'o=s' => sub { add_config_override_opt($config_overrides, $_[1]); },
     'properties:s' => \$getproplist,
