@@ -671,8 +671,10 @@ config_init(0, undef);
 
 my $config_overrides = new_config_overrides($#ARGV+1);
 
+debug("Arguments: " . join(' ', @ARGV));
 Getopt::Long::Configure(qw(bundling));
 GetOptions(
+    'version' => \&Amanda::Util::version_opt,
     'help|usage|?|h' => \&usage,
     'c' => \$opt_only_compression,
     'b=s' => sub {
