@@ -378,22 +378,27 @@ Amanda::MainLoop::run();
 	    is_deeply($inv, [
 	      { slot => 1, state => Amanda::Changer::SLOT_FULL,
 		device_status => $DEVICE_STATUS_DEVICE_ERROR,
+		device_error  => "Error checking directory $taperoot/slot0/data/: No such file or directory",
 		f_type => undef, label => undef,
 		reserved => 0 },
 	      { slot => 2, state => Amanda::Changer::SLOT_FULL,
 		device_status => $DEVICE_STATUS_VOLUME_UNLABELED,
+		device_error  => "File 0 not found",
 		f_type => $Amanda::Header::F_EMPTY, label => undef,
 		reserved => 0 },
 	      { slot => 3, state => Amanda::Changer::SLOT_FULL,
 		device_status => $DEVICE_STATUS_VOLUME_UNLABELED,
+		device_error  => "File 0 not found",
 		f_type => $Amanda::Header::F_EMPTY, label => undef,
 		reserved => 0 },
 	      { slot => 4, state => Amanda::Changer::SLOT_FULL,
 		device_status => $DEVICE_STATUS_SUCCESS,
+		device_error  => undef,
 		f_type => $Amanda::Header::F_TAPESTART, label => "FOO?BAR",
 		reserved => 0, current => 1 },
 	      { slot => 5, state => Amanda::Changer::SLOT_FULL,
 		device_status => $DEVICE_STATUS_VOLUME_UNLABELED,
+		device_error  => "File 0 not found",
 		f_type => $Amanda::Header::F_EMPTY, label => undef,
 		reserved => 0 },
 		], "inventory finds the labeled tape");
