@@ -82,6 +82,7 @@ is_deeply($tl->{'tles'}->[0], {
        'reuse' => 1,
        'barcode' => undef,
        'meta' => undef,
+       'blocksize' => '32',
        'comment' => undef,
        'position' => 1,
        'label' => 'TESTCONF92',
@@ -145,6 +146,7 @@ is_deeply($tl->{'tles'}->[0], {
        'reuse' => 1,
        'barcode' => 'bar-01',
        'meta' => 'meta-01',
+       'blocksize' => undef,
        'comment' => undef,
        'position' => 1,
        'label' => 'TESTCONF88',
@@ -167,6 +169,7 @@ is_deeply($tl->{'tles'}->[0], {
        'reuse' => 1,
        'barcode' => undef,
        'meta' => 'meta-01',
+       'blocksize' => '32',
        'comment' => undef,
        'position' => 1,
        'label' => 'TESTCONF01',
