@@ -16,10 +16,11 @@
 # Contact information: Zmanda Inc, 465 S. Mathilda Ave., Suite 300
 # Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 
-use Test::More tests => 166;
+use Test::More tests => 167;
 
 use strict;
 use warnings;
+use Errno;
 use Cwd qw(abs_path);
 use lib "@amperldir@";
 
@@ -521,6 +522,11 @@ ok(run($amreport, 'TESTCONF', '--from-amdump'),
   or diag($Installcheck::Run::stderr);
 ok(!-f $mail_output, "..produces no mail output");
 is($Installcheck::Run::stdout, "", "..produces no stdout output");
+$! = &Errno::ENOENT;
+my $enoent = $!;
+like($Installcheck::Run::stderr,
+     qr/^error: the mailer '.*' is not an executable program\.$/,
+     "..produces correct stderr output");
 results_match(
     $printer_output,
     $cat->get_text('postscript'),
